@@ -2,7 +2,6 @@ import re
 from typing import *
 import json
 
-
 __author__ = "Fabian Becker <fab.becker@outlook.de>"
 
 
@@ -12,50 +11,61 @@ class Filter:
                     "__lt__": "<",
                     "__gt__": ">",
                     "__lte__": "<=",
-                    "__gte__":  ">=",
+                    "__gte__": ">=",
                     "__contains__": " BETWEEN "}
 
     def __init__(self,
                  operator: str = "AND",
-                 allow_missing_attributes: bool = False,
                  **kwargs):
         self.operator = operator
-        self.allow_missing_attributes = allow_missing_attributes
-        self.ignored_attributes = [x for x in self.__dir__()]
-        self.ignored_attributes.append("ignored_attributes")
+        self.ignored_attributes = [x for x in self.__dir__()] + ["ignored_attributes"]
         # any argument matching the pattern is accepted
         for attr in kwargs:
-            if not (re.match(r'\w+__\w+__$', attr) or attr in self.ignored_attributes or type(kwargs[attr]) == Filter):
+            if not (re.match(r'\S*__\w+__$', attr) or attr in self.ignored_attributes or type(kwargs[attr]) == Filter):
                 raise ValueError(f"{attr} does not match expected attribute format.")
         self.__dict__.update(kwargs)
 
+        assert self.operator in ["AND", "OR"], "An operator has to be chosen."
+
     def __eq__(self, other):
         # check for all/any non hidden attributes of this class
         # if they are present in another object
         # and equal the corresponding attributes of another object
         patterns = [attr for attr in self.__dir__()
-                    if not (attr.startswith("__") or attr in self.ignored_attributes)]
-        checks = []
+                    if attr not in self.ignored_attributes]
+        results = []
         for pattern in patterns:
             if type(self.__getattribute__(pattern)) != Filter:
                 # Check for equality with any object other than filter
+                attributes = [
+                    x for x in pattern.split("__")[0:-2]
+                    if x
+                ]
+                magic_method = "__" + pattern.split("__")[-2] + "__"
+                vo = other
+                r = False
                 try:
-                    attributes = pattern.split("__")[0:-2]
-                    magic_method = "__" + pattern.split("__")[-2] + "__"
-                    vo = other
                     for attr in attributes:
                         vo = vo.__getattribute__(attr)
-                    checks.append(self.__getattribute__(pattern).__getattribute__(magic_method)(vo))
+                    r = vo.__getattribute__(magic_method)(self.__getattribute__(pattern))
                 except AttributeError:
-                    if not self.allow_missing_attributes:
-                        # when attribute is not present in other object, abort
+                    if self.operator == "AND":
+                        # Exit with False result
                         return False
-                    else:
-                        pass
+                    if self.operator == "OR":
+                        # Give another chance without storing the result
+                        continue
+                results.append(bool(r))
             else:
                 # Check for compatibility with filters
-                checks.append(self.__getattribute__(pattern).__eq__(other))
-        return False not in checks and len(checks) >= 1
+                results.append(bool(self.__getattribute__(pattern) == other))
+        if self.operator == "AND":
+            return False not in results
+        if self.operator == "OR":
+            return True in results
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
     def __str__(self, logical_condition: Optional[str] = None):
         # Returns a string to be used for sql style databases
@@ -66,11 +76,11 @@ class Filter:
             value = self.__getattribute__(name)
             use_brackets = (True if type(value) == Filter and len(value) > 1 else False)
             use_quotes = (True if type(value) == str else False)
-            a = (name if type(value) != Filter else "")\
-                + ("'" if use_quotes else "")\
-                + ("(" if use_brackets else "")\
-                + str(value)\
-                + (")" if use_brackets else "")\
+            a = (name if type(value) != Filter else "") \
+                + ("'" if use_quotes else "") \
+                + ("(" if use_brackets else "") \
+                + str(value) \
+                + (")" if use_brackets else "") \
                 + ("'" if use_quotes else "")
             for element in self.replacements:
                 a = a.replace(element, self.replacements[element])
@@ -101,6 +111,5 @@ def filter_from_repr(representation: str) -> Filter:
 
 
 if __name__ == "__main__":
-    from tests import tests
-
-    tests.run_tests()
+    from tests.tests import main
+    main()
